#ifndef __GLOBAL_H__
#define __GLOBAL_H__


#include "config.h"

#define MAIN_CMD_EVENT       0x00000001L

// werden von SL-CAN Treiber benutzt
#define TX_MESSAGES_EVENT    0x00000002L
#define FTDI_EVENT           0x20000000L


#define MAX_LINE_SIZE 200
#define MAX_FILENAME_SIZE 200


#ifndef MAX_CAN_CHANNELS
  #define MAX_CAN_CHANNELS 1
#endif

#ifdef __WIN32__
// ****** Windows
#define ATTRIBUTE_INTERNAL

#elif __APPLE__
// ****** Apple
#define ATTRIBUTE_INTERNAL __attribute((visibility("hidden")))

#else
// ****** Linux
#define ATTRIBUTE_INTERNAL __attribute((visibility("internal")))

#endif

#define ACCESS_PUBLIC     0xFF01
#define ACCESS_PORT_OPEN  0xFF02
#define ACCESS_INIT       0xFF04


#define M_RD_ALL    0xFF00

#define M_WR_PUBLIC 0x0001
#define M_WR_OPEN   0x0003
#define M_WR_INIT   0x0007

#define M_WR_ALL    0x00FF

#define EVENT_RX_MESSAGES         0x0001
#define EVENT_RX_FILTER_MESSAGES  0x0002
#define EVENT_STATUS_CHANGE       0x0004
#define EVENT_PNP_CHANGE          0x0008


#ifdef WINCE

struct TTime
  {
  uint32_t tv_sec;
  uint32_t tv_usec;
  };

#define timeval TTime

#endif

#define safe_min(a,b) (((a) < (b)) ? (a) : (b))
#define safe_strcp(dst, dst_max, src, count) do {memcpy(dst, src, safe_min(count, dst_max)); \
	((char*)dst)[safe_min(count, dst_max)-1] = 0;} while(0)
#define safe_strcpy(dst, dst_max, src) safe_strcp(dst, dst_max, src, strlen(src)+1)

#ifndef __WIN32__
  #define safe_sprintf snprintf
  #define safe_vsprintf vsnprintf
#else
  #if _MSC_VER >= 1500  
    #define safe_sprintf(dst, dst_max, format, ...) _snprintf_s(dst, dst_max, _TRUNCATE, format, __VA_ARGS__)
    #define safe_vsprintf _vsnprintf
  #else
    #define safe_sprintf _snprintf
    #define safe_vsprintf _vsnprintf
  #endif  
#endif


#ifdef __WIN32__
// ****** Windows
#include <windows.h>
//#include <stdint.h>
#ifndef __FUNCTION__
  #define __FUNCTION__ "FUNCTION"
#endif

#ifndef int32_t
  #define int32_t __int32
#endif

#ifndef uint32_t
  #define uint32_t unsigned __int32
#endif

#ifndef int16_t
  #define int16_t __int16
#endif

#ifndef uint16_t
  #define uint16_t unsigned __int16
#endif

#ifndef uint8_t
  #define uint8_t unsigned __int8
#endif

#ifndef int8_t
  #define int8_t __int8
#endif

#define CALLBACK_TYPE CALLBACK
#define DRV_LOCK_TYPE CRITICAL_SECTION
#define DRV_LOCK_INIT(x) do { \
   if (!(x)) \
     { \
     (x) = malloc(sizeof(CRITICAL_SECTION)); \
     InitializeCriticalSection((x)); \
     } \
   } while(0)
#define DRV_LOCK_DESTROY(x) do { \
   if ((x)) \
     { \
     DeleteCriticalSection((x)); \
     free((x)); \
     (x) = NULL; \
     } \
   } while(0)
#define DRV_LOCK_ENTER(x) do {if (x) EnterCriticalSection((x));} while(0)
#define DRV_LOCK_LEAVE(x) do {if (x) LeaveCriticalSection((x));} while(0)
#define mhs_sleep(x) Sleep(x)

#define get_tick() GetTickCount()
#else
// ****** Linux
#include <pthread.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/time.h>
#include <stdint.h>

#define CALLBACK_TYPE
#define DRV_LOCK_TYPE pthread_mutex_t

#define DRV_LOCK_INIT(x) do { \
   if (!(x)) \
     { \
     (x) = malloc(sizeof(pthread_mutex_t)); \
     pthread_mutex_init((x), NULL); \
     } \
   } while(0)
   
#define DRV_LOCK_DESTROY(x) do { \
   if ((x)) \
     { \
     pthread_mutex_destroy((x)); \
     free((x)); \
     (x) = NULL; \
     } \
   } while(0) 
#define DRV_LOCK_ENTER(x) do {if (x)  pthread_mutex_lock((x));} while(0)
#define DRV_LOCK_LEAVE(x) do {if (x)  pthread_mutex_unlock((x));} while(0)
#define mhs_sleep(x) usleep((x) * 1000)


#define FT_DEVICE_BM       0
#define FT_DEVICE_AM       1
#define FT_DEVICE_100AX    2
#define FT_DEVICE_UNKNOWN  3
#define FT_DEVICE_2232C    4
#define FT_DEVICE_232R     5
#define FT_DEVICE_2232H    6
#define FT_DEVICE_4232H    7
#define FT_DEVICE_232H     8
#define FT_DEVICE_X_SERIES 9


#endif

/*#ifdef __WIN32__
// ****** Windows

#else
// ****** Linux
#endif*/

#include "os_mhstcan.h"

#include "can_types.h"
#include "drv_types.h"
#include "tcan_drv.h"

#endif
