/***************************************************************************
                        tcan_drv.c  -  description
                             -------------------
    begin             : 18.02.2020
    last modify       : 18.02.2020
    copyright         : (C) 2020 by MHS-Elektronik GmbH & Co. KG, Germany
    author            : Klaus Demlehner, klaus@mhs-elektronik.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software, you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License           *
 *   version 2.1 as published by the Free Software Foundation.             *
 *                                                                         *
 ***************************************************************************/

/**
    Library to talk to Tiny-CAN devices. You find the latest versions at
       http://www.tiny-can.com/
**/
#include "global.h"
#include "util.h"
#include "fd_speeds.h"

struct TCanSpeedBtr
  {
  uint16_t Speed;
  uint32_t Btr;
  };

// Speed kBit/s | DBTR Value
static const struct TCanSpeedBtr CanFdSpeedTab[] = {
  {250,          0x000B0A33},  //  250000Bit/s (0.0% /  250000Bit/s)  PH1=11   PH2=4    SP=75.0%  SJW=4   
  {500,          0x00050A33},  //  500000Bit/s (0.0% /  500000Bit/s)  PH1=11   PH2=4    SP=75.0%  SJW=4    
  {1000,         0x00020A33},  // 1000000Bit/s (0.0% / 1000000Bit/s)  PH1=11   PH2=4    SP=75.0%  SJW=4    
  {1500,         0x00010A33},  // 1500000Bit/s (0.0% / 1500000Bit/s)  PH1=11   PH2=4    SP=75.0%  SJW=4     
  {2000,         0x00001053},  // 2000000Bit/s (0.0% / 2000000Bit/s)  PH1=17   PH2=6    SP=75.0%  SJW=4    
  {3000,         0x00000A33},  // 3000000Bit/s (0.0% / 3000000Bit/s)  PH1=11   PH2=4    SP=75.0%  SJW=4    
  {4000,         0x00000633},  // 4000000Bit/s (0.0% / 4000000Bit/s)  PH1=7    PH2=4    SP=66.6%  SJW=4  
  {0,            0}};
  
  
uint32_t GetDBtrValueFromSpeed(TCanDevice *dev, uint16_t fd_speed)
{
const struct TCanSpeedBtr *tab;
uint16_t s;

for (tab = &CanFdSpeedTab[0]; (s = tab->Speed); tab++)
  {
  if (s == fd_speed)
    return(tab->Btr);
  }
return(0);
}  
