#ifndef __TCAN_DRV_H__
#define __TCAN_DRV_H__

#include "global.h"  // Definition von ATTRIBUTE_INTERNAL
#include "com_io.h"
#include "var_obj.h"
#include "mhs_thread.h"
#include "mhs_event_list.h"

#ifdef __cplusplus
  extern "C" {
#endif

#define CAN_FEATURE_LOM          0x0001  // Silent Mode (LOM = Listen only Mode)
#define CAN_FEATURE_ARD          0x0002  // Automatic Retransmission disable
#define CAN_FEATURE_TX_ACK       0x0004  // TX ACK (Gesendete Nachrichten besttigen)
#define CAN_FEATURE_ERROR_MSGS   0x0008  // Error Messages Support
#define CAN_FEATURE_FD_HARDWARE  0x0010  // CAN-FD Hardware
#define CAN_FEATURE_FIFO_OV_MODE 0x0020  // FIFO OV Mode (Auto Clear, OV CAN Messages)
#define CAN_FEATURE_HW_TIMESTAMP 0x8000  // Hardware Time Stamp


#define DEVICE_NAME_MAX_SIZE   255
#define SERIAL_NUMBER_MAX_SIZE 16 
#define DESCRIPTION_MAX_SIZE   64

#define TIME_STAMP_OFF        0
#define TIME_STAMP_SOFT       1
#define TIME_STAMP_HW_UNIX    2
#define TIME_STAMP_HW         3
#define TIME_STAMP_HW_SW_UNIX 4

#define MCMD_DEVICE_CLOSE     1
#define MCMD_DEVICE_OPEN      2
#define MCMD_CAN_STOP         4
#define MCMD_CAN_START        5
#define MCMD_CAN_RESET        6
#define MCMD_CAN_ONLY_CMD     12

#define MCMD_CAN_START_LOM 14
#define MCMD_CAN_START_NO_RETRANS 15
#define MCMD_CAN_SET_TX_ACK_ENABLE 16

#define DEV_LIST_SHOW_TCAN_ONLY 0x01
#define DEV_LIST_SHOW_UNCONNECT 0x02


#define TINY_CAN_TYPE_I_XL    1  
#define TINY_CAN_TYPE_II_XL   2
#define TINY_CAN_TYPE_III_XL  3
#define TINY_CAN_TYPE_M1      4
#define TINY_CAN_TYPE_M232    5
#define TINY_CAN_TYPE_IV_XL   6
#define TINY_CAN_TYPE_LS      7
#define TINY_CAN_TYPE_M2      8
#define TINY_CAN_TYPE_M3      9


#pragma pack(push, 1)
struct TModulFeatures  // In tcan_drv.h kopiert
  {
  uint32_t CanClock;
  uint32_t Flags;
  uint32_t CanChannelsCount;
  uint32_t HwRxFilterCount;
  uint32_t HwTxPufferCount;
  };
#pragma pack(pop)  


#pragma pack(push, 1)
struct TCanDevicesList
  {
  uint32_t TCanIdx;
  uint32_t HwId;
  char DeviceName[DEVICE_NAME_MAX_SIZE];
  char SerialNumber[SERIAL_NUMBER_MAX_SIZE];
  char Description[DESCRIPTION_MAX_SIZE];
  struct TModulFeatures ModulFeatures;
  };
#pragma pack(pop)  


#pragma pack(push, 1)
struct TCanDeviceInfo
  {
  uint32_t HwId;
  uint32_t FirmwareVersion;
  uint32_t FirmwareInfo;
  char SerialNumber[SERIAL_NUMBER_MAX_SIZE];
  char Description[DESCRIPTION_MAX_SIZE];
  struct TModulFeatures ModulFeatures;
  };
#pragma pack(pop)


struct TCmdSetModeParam
  {
  uint32_t Index;
  uint16_t CanCommand;
  };


#pragma pack(push, 1)
struct TInfoVar
  {
  uint32_t Key;
  uint32_t Type; 
  uint32_t Size;
  char Data[255];
  };
#pragma pack(pop)


struct TInfoVarList
  {
  struct TInfoVarList *Next;
  uint8_t Size;
  uint8_t Type;
  uint16_t Index;
  const char *IndexString;
  char *Data;
  };

struct TCanModulDesc
  {
  uint32_t HwId;
  char *Description;
  uint32_t BaudRate[2];
  uint32_t Flags;
  uint32_t StatusSize;
  uint32_t SupportHwTimestamp;
  uint32_t SupportTxAck;
  uint32_t SupportCanErrorMsgs;
  uint32_t CanFeaturesFlags;
  uint32_t CanChannels;
  uint32_t DigIoCount;
  uint32_t HwTxFifoSize;
  uint32_t HwTxFifoLimit;
  uint32_t HwRxFilterCount;
  uint32_t HwTxPufferCount;
  uint32_t CanClock;
  uint32_t RxMtu;
  uint32_t TinyCanType;
  };

#define SM_DRV_PARAMETER_COUNT 4096

struct TSMDrvParameter
  {
  uint8_t Data[SM_DRV_PARAMETER_COUNT];
  uint32_t Count;
  };

typedef struct _TMhsCmd TMhsCmd;

struct _TMhsCmd
  {
  TMhsEvent Event;
#ifdef __WIN32__  
  CRITICAL_SECTION CmdLock;
#else  
  pthread_mutex_t CmdLock;
#endif  
  uint32_t Command;
  uint32_t ParamCount;
  char *Param;
  uint32_t Result;
  };

typedef struct _TCanDevice TCanDevice;

struct _TCanDevice
  {
  // **** Device Index
  uint32_t Index;
  // **** Reference Counter
  volatile uint32_t RefCount;
  // **** Context
  TMhsObjContext *IoContext;
  TMhsObjContext *Context;
  TMhsObjContext *TxPufferContext;
  TMhsObjContext *RxFilterContext;
  // **** Kommando
  TMhsCmd *Cmd;
  // **** Main Thread
  TMhsThread *MainThread;
  struct TIo *Io;
  // **** Modul Beschreibung  
  struct TCanModulDesc *ModulDesc;
  struct TInfoVarList *InfoVars;
  char *DrvInfoHwStr;
  uint32_t HaveRtsCtsClear;
  uint32_t Protokoll;
  // **** Uebergabepuffer fr Callback-Funktion
  struct TCanMsg *CanRxDBuffer;
  uint32_t CanRxDBufferSize;

  // **** Variablen
  uint32_t TimeStampMarker;
  uint8_t UseDigIo;
  uint32_t TimeNow;
  uint32_t StatusTimeStamp;
  uint32_t FilterTimeStamp;
  uint32_t HwTxDFifoCount[MAX_CAN_CHANNELS];
  uint32_t ReadStatusFlag;
  struct TCanFdMsg TxCanBuffer[HW_TX_FIFO_MAX];  
  uint32_t CommIdleCounter;
  
  uint8_t DigOutSet;
  uint16_t DigOut[MAX_IO_CFG/2];
  uint8_t AnalogOutSet;  
  uint16_t AnalogOut[4];
  // **** Konfigurationsvariablen
  uint32_t TxCanFifoEventLimit;  
  uint8_t TimeStampMode;
  uint32_t FilterReadIntervall;
  uint32_t HighPollIntervall;
  uint32_t IdlePollIntervall;
  uint8_t CommTryCount;  
  // **** Device Status
  struct TDeviceStatus DeviceStatus;
  struct TDeviceStatus LastDeviceStatus;
  int32_t StatusChange;
  TMhsEventList *StatusEventList;
  // **** SM
  uint8_t TxDDataPuffer[SM_DRV_PARAMETER_COUNT+4];
  uint8_t Kommando;
  struct TSMDrvParameter TxDParameter;  // Sende Parameter
  struct TSMDrvParameter RxDParameter;  // Empfangene Parameter
  uint8_t ACKByte;                // Empfangenes ACK Byte
  uint8_t CanFd;
  uint16_t OvMode;
  };


extern const struct TCanModulDesc ExCanModulTable[] ATTRIBUTE_INTERNAL; 

TCanDevice *CreateCanDevice(void) ATTRIBUTE_INTERNAL;
void DestroyCanDevice(TCanDevice **device) ATTRIBUTE_INTERNAL;
int32_t GetDeviceInfo(TCanDevice *dev, struct TCanDeviceInfo *device_info) ATTRIBUTE_INTERNAL;
int32_t OpenCanDevice(TCanDevice *dev) ATTRIBUTE_INTERNAL;
void CloseCanDevice(TCanDevice *dev) ATTRIBUTE_INTERNAL;
int32_t SetIntervallBox(TCanDevice *dev, uint32_t index, uint16_t cmd, uint32_t time) ATTRIBUTE_INTERNAL;

void DevStatusEventSetup(TCanDevice *dev, TMhsEvent *event, uint32_t events) ATTRIBUTE_INTERNAL;

#ifdef __cplusplus
  }
#endif

#endif
